package cn.sliyun.api.member.service.impl;

import cn.sliyun.common.constant.Constants;
import cn.sliyun.common.exception.AppException;
import cn.sliyun.common.utils.IdUtil;
import cn.sliyun.common.utils.RegUtil;
import cn.sliyun.api.member.entity.Member;
import cn.sliyun.api.member.mapper.MemberMapper;
import cn.sliyun.api.member.service.IMemberService;
import cn.sliyun.api.user.entity.UpdateUserEntity;
import cn.sliyun.api.user.entity.UserActionBean;
import cn.sliyun.common.core.Result;
import cn.sliyun.common.core.ResultGenerator;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.HashMap;
import java.util.Map;

/**
 * <p>
 * 会员表 服务实现类
 * </p>
 *
 * @author Enoch
 * @since 2021-02-24
 */
@Service
public class MemberServiceImpl extends ServiceImpl<MemberMapper, Member> implements IMemberService {


    public Result register(UserActionBean userRegister) {
        if (!RegUtil.checkMobile(userRegister.getPhone())) {
            throw new AppException("手机号不正确");
        }
        System.out.println("phone:" + userRegister.getPhone());
        Member user = baseMapper.selectOne(new QueryWrapper<Member>().eq("phone", userRegister.getPhone()));
        if (user != null) {
            System.out.println("phone:" + user.getUsername() + "/" + user.getPhone());
            throw new AppException("手机号已经注册");
        }

        Member member = new Member();
        member.setMemberSn("" + IdUtil.create());
        member.setNickname(StringUtils.isEmpty(userRegister.getName()) ? IdUtil.genNickname() : userRegister.getName() + "");
        member.setUsername(StringUtils.isEmpty(userRegister.getName()) ? IdUtil.genNickname() : userRegister.getName() + "");

        if (StringUtils.isEmpty(userRegister.getOssFile())) {
            member.setAvatarOssFile(Constants.IMG_DEFAULT_USER_AVATAR);
        } else {
            member.setAvatarOssFile(userRegister.getOssFile());
        }
        member.setPhone(userRegister.getPhone());

        member.setPassword(userRegister.getPassword());
        member.setCreateTime(LocalDateTime.now());
        member.setUpdateTime(LocalDateTime.now());
        member.setSourceType(1);
        int id = baseMapper.insert(member);
        return ResultGenerator.genSuccessResult(id);
    }

    public Result edit(String token, UpdateUserEntity updateUser) {
        Member user = baseMapper.selectOne(new QueryWrapper<Member>().ge("token", token));
        if (user == null) {
            return ResultGenerator.genFailResult("用户不存在");
        }
        BeanUtils.copyProperties(updateUser, user);
        baseMapper.update(user, new UpdateWrapper<>());
        return ResultGenerator.genSuccessResult();
    }

    public Result getInfo(String token) {
        Member user = baseMapper.selectOne(new QueryWrapper<Member>().eq("token", token));
        if (user == null) {
            throw new AppException("用户不存在");
        }
        return ResultGenerator.genSuccessResult(user);
    }


    public Result updatePassword(UserActionBean updateUser) {
        Member user = baseMapper.selectOne(new QueryWrapper<Member>().eq("phone", updateUser.getPhone()));
        if (user == null) {
            throw new AppException("用户不存在");
        }
        user.setPassword(updateUser.getPassword());
        baseMapper.update(user, new UpdateWrapper<>());
        return ResultGenerator.genSuccessResult(user);
    }

    public Result getList(Integer pageIndex, Integer size) {
        System.out.println(pageIndex + "/" + size);
        Page<Member> page = new Page<>(pageIndex, size);
        QueryWrapper queryWrapper = new QueryWrapper();
        queryWrapper.eq("is_delete", 0);
        Page<Member> userMyPage = baseMapper.selectPage(page, queryWrapper);
        Map map = new HashMap();
        map.put("totalCount", userMyPage.getTotal());
        map.put("totalPage", userMyPage.getPages());
        map.put("pageNo", userMyPage.getCurrent());
        map.put("pageSize", size);
        map.put("data", userMyPage.getRecords());
        return ResultGenerator.genSuccessResult(map);
    }


    public Result updateStatus(Map<String, Object> params) {
        Member user = baseMapper.selectOne(new QueryWrapper<Member>().eq("id", params.get("id")));
        if (user == null) {
            throw new AppException("用户不存在");
        }
        user.setStatus((Integer) params.get("status"));
        baseMapper.updateById(user);
        return ResultGenerator.genSuccessResult();
    }

    public Result onDelete(Map<String, Object> params) {
        Member user = baseMapper.selectOne(new QueryWrapper<Member>().eq("id", params.get("id")));
        if (user == null) {
            throw new AppException("用户不存在");
        }
        user.setIsDelete(1);
        baseMapper.updateById(user);
        return ResultGenerator.genSuccessResult();
    }


    public Member getLoginUserInfo(String token) {
        Member member = baseMapper.selectOne(new QueryWrapper<Member>().eq("token", token));
        if (member == null) {
            throw new AppException("请先登录");
        }

        return member;
    }

    public long getCount() {
        return count();
    }


}
