package cn.sliyun.api.order.entity;

import java.math.BigDecimal;

import cn.sliyun.common.utils.AliyunOSSUtil;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableName;
import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableId;
import com.fasterxml.jackson.annotation.JsonIgnore;

import java.time.LocalDateTime;
import java.io.Serializable;

/**
 * <p>
 * 订单表
 * </p>
 *
 * @author Enoch
 * @since 2021-02-24
 */
@TableName("sly_order")
public class Order implements Serializable {


    private static final long serialVersionUID = 1L;

    /**
     * 订单id
     */
    @TableId(value = "id", type = IdType.AUTO)
    private Long id;

    private Integer memberId;


    /**
     * 订单编号
     */
    private String orderSn;

    /**
     * 用户帐号
     */
    private String memberSn;

    /**
     * 订单总金额
     */
    private BigDecimal totalAmount;

    /**
     * 应付金额（实际支付金额）
     */
    private BigDecimal payAmount;

    /**
     * 管理员后台调整订单使用的折扣金额
     */
    @JsonIgnore
    private BigDecimal discountAmount;

    /**
     * 支付方式：0->未支付；1->支付宝；2->微信
     */
    private Integer payType=0;

    @TableField(exist = false)
    private String payTypeName;

    /**
     * 订单来源：0->PC订单；1->app订单; 2> web 订单
     */
    private Integer sourceType;

    @TableField(exist = false)
    private String sourceTypeName;

    /**
     * 订单状态：0->待付款；1->待进行；2->进行中；3->已完成；4->已关闭；5->无效订单
     */
    private Integer status;


    @TableField(exist = false)
    private String statusName;

    /**
     * 订单类型：0->正常订单；
     */
    private Integer orderType;


    /**
     * 货币ID
     */
    private Integer coinId;

    /**
     * 钱包地址
     */
    private Long coinAddressId;

    /**
     * 订单图片
     */
    @JsonIgnore
    private String ossImg;

    /**
     * 购买时长
     */
    private long duration;

    /**
     * 规格包
     */
    private Integer sukId;

    /**
     * 租赁状态（0 待进行 1 进行中 2 已完成 ）
     */
    private String leaseStatus;

    /**
     * 租用开始时间
     */
    private LocalDateTime leaseStartTime;

    /**
     * 租用结束时间
     */
    private LocalDateTime leaseEndTime;

    /**
     * 支付时间
     */
    private LocalDateTime paymentTime;

    /**
     * 创建时间
     */
    private LocalDateTime createTime;

    /**
     * 更新时间
     */
    private LocalDateTime updateTime;

    /**
     * remarks
     */
    private String remarks;


    @TableField(exist = false)
    public String icon;


    public String getStatusName() {
        if (status == 0) {
            return "待付款";
        } else if (status == 1) {
            return "待进行";
        } else if (status == 2) {
            return "进行中";
        } else if (status == 3) {
            return "已完成";
        } else if (status == 4) {
            return "已关闭";
        } else {
            return "无效订单";
        }
    }

    public String getPayTypeName() {
        if (payType == 0) {
            return "未支付";
        } else if (payType == 1) {
            return "支付宝";
        } else if (payType == 2) {
            return "微信";
        } else if (payType == 3) {
            return "后台确认";
        } else {
            return "未支付";
        }
    }

    public String getSourceTypeName() {
        if (orderType == 0) {
            return "PC";
        } else if (status == 1) {
            return "APP";
        } else if (status == 2) {
            return "H5";
        }
        return "未知来源";
    }

    public String getIcon() {
        return AliyunOSSUtil.getInstance().getFileUrl(ossImg);
    }

    public void setIcon(String icon) {
        this.icon = icon;
    }

    public Long getId() {
        return id;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public Integer getMemberId() {
        return memberId;
    }

    public void setMemberId(Integer memberId) {
        this.memberId = memberId;
    }

    public String getOrderSn() {
        return orderSn;
    }

    public void setOrderSn(String orderSn) {
        this.orderSn = orderSn;
    }


    public String getMemberSn() {
        return memberSn;
    }

    public void setMemberSn(String memberSn) {
        this.memberSn = memberSn;
    }

    public BigDecimal getTotalAmount() {
        return totalAmount;
    }

    public void setTotalAmount(BigDecimal totalAmount) {
        this.totalAmount = totalAmount;
    }

    public BigDecimal getPayAmount() {
        return payAmount;
    }

    public void setPayAmount(BigDecimal payAmount) {
        this.payAmount = payAmount;
    }

    public BigDecimal getDiscountAmount() {
        return discountAmount;
    }

    public void setDiscountAmount(BigDecimal discountAmount) {
        this.discountAmount = discountAmount;
    }

    public Integer getPayType() {
        return payType;
    }

    public void setPayType(Integer payType) {
        this.payType = payType;
    }

    public Integer getSourceType() {
        return sourceType;
    }

    public void setSourceType(Integer sourceType) {
        this.sourceType = sourceType;
    }

    public Integer getStatus() {
        return status;
    }

    public void setStatus(Integer status) {
        this.status = status;
    }

    public Integer getOrderType() {
        return orderType;
    }

    public void setOrderType(Integer orderType) {
        this.orderType = orderType;
    }

    public Integer getCoinId() {
        return coinId;
    }

    public void setCoinId(Integer coinId) {
        this.coinId = coinId;
    }

    public Long getCoinAddressId() {
        return coinAddressId;
    }

    public void setCoinAddressId(Long coinAddressId) {
        this.coinAddressId = coinAddressId;
    }

    public String getOssImg() {
        return ossImg;
    }

    public void setOssImg(String ossImg) {
        this.ossImg = ossImg;
    }

    public long getDuration() {
        return duration;
    }

    public void setDuration(long duration) {
        this.duration = duration;
    }

    public Integer getSukId() {
        return sukId;
    }

    public void setSukId(Integer suk) {
        this.sukId = suk;
    }

    public String getLeaseStatus() {
        return leaseStatus;
    }

    public void setLeaseStatus(String leaseStatus) {
        this.leaseStatus = leaseStatus;
    }

    public LocalDateTime getLeaseStartTime() {
        return leaseStartTime;
    }

    public void setLeaseStartTime(LocalDateTime leaseStartTime) {
        this.leaseStartTime = leaseStartTime;
    }

    public LocalDateTime getLeaseEndTime() {
        return leaseEndTime;
    }

    public void setLeaseEndTime(LocalDateTime leaseEndTime) {
        this.leaseEndTime = leaseEndTime;
    }

    public LocalDateTime getPaymentTime() {
        return paymentTime;
    }

    public void setPaymentTime(LocalDateTime paymentTime) {
        this.paymentTime = paymentTime;
    }

    public LocalDateTime getCreateTime() {
        return createTime;
    }

    public void setCreateTime(LocalDateTime createTime) {
        this.createTime = createTime;
    }

    public LocalDateTime getUpdateTime() {
        return updateTime;
    }

    public void setUpdateTime(LocalDateTime updateTime) {
        this.updateTime = updateTime;
    }

    public String getRemarks() {
        return remarks;
    }

    public void setRemarks(String remarks) {
        this.remarks = remarks;
    }

    @Override
    public String toString() {
        return "Order{" +
                "id=" + id +
                ", memberId=" + memberId +
                ", orderSn=" + orderSn +
                ", memberSn=" + memberSn +
                ", totalAmount=" + totalAmount +
                ", payAmount=" + payAmount +
                ", discountAmount=" + discountAmount +
                ", payType=" + payType +
                ", sourceType=" + sourceType +
                ", status=" + status +
                ", orderType=" + orderType +
                ", coinId=" + coinId +
                ", coinAddress=" + coinAddressId +
                ", ossImg=" + ossImg +
                ", duration=" + duration +
                ", sukId=" + sukId +
                ", leaseStatus=" + leaseStatus +
                ", leaseStartTime=" + leaseStartTime +
                ", leaseEndTime=" + leaseEndTime +
                ", paymentTime=" + paymentTime +
                ", createTime=" + createTime +
                ", updateTime=" + updateTime +
                ", remarks=" + remarks +
                "}";
    }
}
