package cn.sliyun.api.user.service.impl;

import cn.sliyun.common.constant.Constants;
import cn.sliyun.common.exception.AppException;
import cn.sliyun.common.utils.IdUtil;
import cn.sliyun.common.utils.RegUtil;
import com.alibaba.fastjson.JSON;
import cn.sliyun.api.user.entity.BeamUser;
import cn.sliyun.api.user.entity.UpdateUserEntity;
import cn.sliyun.api.user.entity.UserActionBean;
import cn.sliyun.api.user.mapper.BeamUserMapper;
import cn.sliyun.api.user.service.IBeamUserService;
import cn.sliyun.common.core.Result;
import cn.sliyun.common.core.ResultGenerator;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

import java.io.Serializable;
import java.time.LocalDateTime;
import java.util.HashMap;
import java.util.Map;

/**
 * <p>
 * 用户表 服务实现类
 * </p>
 *
 * @author Enoch
 * @since 2021-01-28
 */
@Service
public class BeamUserServiceImpl extends ServiceImpl<BeamUserMapper, BeamUser> implements IBeamUserService {


    public Result register(UserActionBean userRegister) {
        if (!RegUtil.checkMobile(userRegister.getPhone())) {
            throw new AppException("手机号不正确");
        }
        System.out.println("phone:"+userRegister.getPhone());
        BeamUser user = baseMapper.selectOne(new QueryWrapper<BeamUser>().eq("phone", userRegister.getPhone()));
        if (user != null) {
            System.out.println("phone:"+user.getName()+"/"+user.getPhone());
            throw new AppException("手机号已经注册");
        }

        BeamUser beamUser = new BeamUser();
        beamUser.setAccount("" + IdUtil.create());
        beamUser.setName(StringUtils.isEmpty(userRegister.getName()) ? "会员" + IdUtil.genNickname() : userRegister.getName() + "");
        beamUser.setAvatar(Constants.IMG_DEFAULT_USER_AVATAR);
        beamUser.setPhone(userRegister.getPhone());
        beamUser.setSalt(userRegister.getPhone());
        beamUser.setPassword(userRegister.getPassword());
        beamUser.setCreatedTime(LocalDateTime.now());
        beamUser.setCreatedBy(userRegister.getPhone());
        beamUser.setUpdateTime(LocalDateTime.now());
        beamUser.setUpdateBy(userRegister.getPhone());
        int id = baseMapper.insert(beamUser);
        return ResultGenerator.genSuccessResult(id);
    }

    public Result edit(String token, UpdateUserEntity updateUser) {
        BeamUser user = baseMapper.selectOne(new QueryWrapper<BeamUser>().ge("token", token));
        if (user == null) {
            return ResultGenerator.genFailResult("用户不存在");
        }
        BeanUtils.copyProperties(updateUser, user);
        baseMapper.update(user, new UpdateWrapper<>());
        return ResultGenerator.genSuccessResult();
    }

    public Result getInfo(String token) {
        BeamUser user = baseMapper.selectOne(new QueryWrapper<BeamUser>().eq("token", token));
        if (user == null) {
            throw  new AppException("用户不存在");
        }
        return ResultGenerator.genSuccessResult(user);
    }


    public Result updatePassword(UserActionBean updateUser) {
        BeamUser user = baseMapper.selectOne(new QueryWrapper<BeamUser>().eq("phone", updateUser.getPhone()));
        if (user == null) {
            throw  new AppException("用户不存在");
        }
        user.setPassword(updateUser.getPassword());
        baseMapper.update(user, new UpdateWrapper<>());
        return ResultGenerator.genSuccessResult(user);
    }

    public Result getList(Integer pageIndex, Integer size) {
        System.out.println(pageIndex + "/" + size);
        Page<BeamUser> page = new Page<>(pageIndex, size);
        Page<BeamUser> userMyPage = baseMapper.selectPage(page, new QueryWrapper<>());
        System.out.println("" + JSON.toJSONString(userMyPage));
        Map map = new HashMap();
        map.put("totalCount", userMyPage.getTotal());
        map.put("totalPage", userMyPage.getPages());
        map.put("pageNo", userMyPage.getCurrent());
        map.put("pageSize", size);
        map.put("data", userMyPage.getRecords());
        return ResultGenerator.genSuccessResult(map);
    }


    public Result updateStatus(Map<String, Object> params) {
        BeamUser user = baseMapper.selectOne(new QueryWrapper<BeamUser>().eq("id", params.get("id")));
        if (user == null) {
            throw  new AppException("用户不存在");
        }
        user.setStatus(params.get("status").toString());
        baseMapper.updateById(user);
        return ResultGenerator.genSuccessResult();
    }
}
