package cn.sliyun.common.utils;

import cn.sliyun.framework.config.OSSConfig;
import com.aliyun.oss.OSS;
import com.aliyun.oss.OSSClient;
import com.aliyun.oss.OSSClientBuilder;
import com.aliyun.oss.OSSException;
import com.aliyun.oss.model.CannedAccessControlList;
import com.aliyun.oss.model.CreateBucketRequest;
import com.aliyun.oss.model.PutObjectRequest;
import com.aliyun.oss.model.PutObjectResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.UUID;

public class AliyunOSSUtil {

    private static AliyunOSSUtil instance = null;


    public static AliyunOSSUtil getInstance() {
        if (null == instance) {
            instance = new AliyunOSSUtil();
        }
        return instance;
    }

    public AliyunOSSUtil() {
        init();
    }

    public static String upload(File file) {
        return upload(file, OSSConfig.OSS_FILE_HOST);
    }


    public static String uploadApk(File file) {
        return upload(file, OSSConfig.OSS_FILE_HOST);
    }

    public static String uploadImg(File file) {
        return upload(file, "img");
    }

    public static String upload(File file, String path) {
        Logger logger = LoggerFactory.getLogger(AliyunOSSUtil.class);
        String endpoint = OSSConfig.OSS_END_POINT;
        String accessKeyId = OSSConfig.OSS_ACCESS_KEY_ID;
        String accessKeySecret = OSSConfig.OSS_ACCESS_KEY_SECRET;
        String bucketName = OSSConfig.OSS_BUCKET_NAME;
        String fileHost = path;

        SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");
        String dateStr = format.format(new Date());
        if (null == file) {
            return null;
        }
        OSSClient ossClient = new OSSClient(endpoint, accessKeyId, accessKeySecret);
        try {
            //容器不存在，就创建
            if (!ossClient.doesBucketExist(bucketName)) {
                ossClient.createBucket(bucketName);
                CreateBucketRequest createBucketRequest = new CreateBucketRequest(bucketName);
                createBucketRequest.setCannedACL(CannedAccessControlList.PublicRead);
                ossClient.createBucket(createBucketRequest);
            }

            //创建文件路径
            String fileUrl = fileHost + "/" + (dateStr + "/" + UUID.randomUUID().toString().replace("-", "") + "-");

            String fileName = URLDecoder.decode(file.getName().toString(), "UTF-8").replaceAll(" ", "");

            fileUrl = fileUrl + fileName;
            //上传文件
            PutObjectResult result = ossClient.putObject(new PutObjectRequest(bucketName, fileUrl, file));
            //设置权限 这里是公开读
            ossClient.setBucketAcl(bucketName, CannedAccessControlList.PublicRead);
            if (null != result) {

                return "https://" + bucketName + "." + endpoint + "/" + fileUrl;
            }
        } catch (OSSException | UnsupportedEncodingException oe) {
            logger.error(oe.toString());
        } finally {
            logger.error("ossClient  finally");
            //关闭
            ossClient.shutdown();
        }
        return null;
    }



    public  String uploadImage(MultipartFile file) {
        Logger logger = LoggerFactory.getLogger(AliyunOSSUtil.class);
        String endpoint = OSSConfig.OSS_END_POINT;
        String accessKeyId = OSSConfig.OSS_ACCESS_KEY_ID;
        String accessKeySecret = OSSConfig.OSS_ACCESS_KEY_SECRET;
        String bucketName = OSSConfig.OSS_BUCKET_NAME;
        String fileHost = "img";

        SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");
        String dateStr = format.format(new Date());
        if (null == file) {
            return null;
        }
        OSSClient ossClient = new OSSClient(endpoint, accessKeyId, accessKeySecret);
        try {
            //容器不存在，就创建
            if (!ossClient.doesBucketExist(bucketName)) {
                ossClient.createBucket(bucketName);
                CreateBucketRequest createBucketRequest = new CreateBucketRequest(bucketName);
                createBucketRequest.setCannedACL(CannedAccessControlList.PublicRead);
                ossClient.createBucket(createBucketRequest);
            }

            String fileUrl = fileHost + "/";
            String oldName = file.getOriginalFilename();
            System.out.println("-----------文件原始的名字【" + oldName + "】-----------");
            String newName = "img_coin_" + System.currentTimeMillis() + oldName.substring(oldName.lastIndexOf("."));
            System.out.println("-----------文件要保存后的新名字【" + newName + "】-----------");
            String fileName = URLDecoder.decode(newName, "UTF-8").replaceAll(" ", "");

            fileUrl = fileUrl + fileName;
            //上传文件
            InputStream multipartFileInputStream = file.getInputStream();

            PutObjectResult result = ossClient.putObject(new PutObjectRequest(bucketName, fileUrl, multipartFileInputStream));
            //设置权限 这里是公开读
            ossClient.setBucketAcl(bucketName, CannedAccessControlList.PublicRead);
            if (null != result) {
                return newName;
            }
        } catch (OSSException | UnsupportedEncodingException oe) {
            logger.error(oe.toString());
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            logger.error("ossClient  finally");
            //关闭
            ossClient.shutdown();
        }
        return null;
    }


    OSS ossClient;

    private void init() {
        ossClient = new OSSClientBuilder().build(OSSConfig.OSS_END_POINT, OSSConfig.OSS_ACCESS_KEY_ID, OSSConfig.OSS_ACCESS_KEY_SECRET);

    }


    public String getFileUrl(String objectName) {
        objectName = OSSConfig.OSS_IMG_PATH + objectName;
        return OSSConfig.OSS_DOMAIN + objectName;    }

}
